# -*- coding: utf-8 -*-

"""
Copyright: LAViD - Laboratório de Aplicações de Vídeo Digital
Author: André Araújo <andre.araujo@lavid.ufpb.br>
File: vlibras.py
Date: 21/02/2017
Version: 1.0.0
"""

import globalPluginHandler
import gui
import speechViewer
import synthDriverHandler
import synthDrivers
import ui

import logging
import os
import socket
import threading
import time


RELEASE = True


class Logger(object):

    def __init__(self, **kwargs):
        object.__init__(self)
        self.__logname = str(kwargs.get("logname", __name__))
        self.__logwrite = bool(kwargs.get("logwrite", True))
        self.__loglevel = int(kwargs.get("loglevel", logging.NOTSET))
        self.__logpath = os.path.splitext(__file__)[0] + ".log"
        self.__logformat = "[%(asctime)s] %(levelname)s %(funcName)s %(lineno)s: %(message)s"
        self.log = logging.getLogger(self.__logname)
        self.log.setLevel(self.__loglevel)
        if (self.__logwrite):
            self.filehandler = logging.FileHandler(self.__logpath)
            self.filehandler.setFormatter(logging.Formatter(self.__logformat))
            self.filehandler.setLevel(self.__loglevel)
            self.log.addHandler(self.filehandler)
        # logging.basicConfig(format=self.__logformat)


logHandler = None

if (RELEASE):
    logHandler = Logger(loglevel=logging.INFO)
else:
    logHandler = Logger(loglevel=logging.DEBUG)


class TCPServer(threading.Thread):

    global logHandler

    def __init__(self, addr):
        threading.Thread.__init__(self)
        self.__addr = addr
        self.__mutexAcc = threading.Lock()
        self.__mutexSend = threading.Lock()
        self.__sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.__sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.__sock.settimeout(None)
        self.__cliSock = None
        self.__cliAddr = None
        self.__cliAlive = False

    def connected(self):
        return self.__cliAlive

    def send(self, message=""):
        self.__mutexSend.acquire()
        try:
            if (self.__cliAlive):
                try:
                    message = message.encode("utf-8", "ignore")
                    if (self.__cliSock.sendto(message, self.__cliAddr)):
                        logHandler.log.debug("Sent to [%s:%s]: %s" % (self.__cliAddr[0], self.__cliAddr[1], message))
                except:
                    logHandler.log.info("Connection canceled by client: [%s:%s]" % (self.__cliAddr[0], self.__cliAddr[1]))
                    self.__cliAlive = False
                    self.__mutexAcc.release()
        finally:
            self.__mutexSend.release()

    def run(self):
        logHandler.log.info("VLibras plugin started")
        try:
            self.__sock.bind(self.__addr)
            self.__sock.listen(5)
        except:
            msg = (u"Outra aplicação está impedindo o VLibras plugin de ser executado")
            logHandler.log.error("Address in use: [%s:%d], exiting..." % (self.__addr[0], self.__addr[1]))
            logHandler.log.error(msg)
            ui.message(msg)
            return
        while True:
            logHandler.log.info("Waiting connection at [%s:%d]" % (self.__addr[0], self.__addr[1]))
            tmpCliSock, tmpCliAddr = self.__sock.accept()
            logHandler.log.info("New client connected: [%s:%d]" % (tmpCliAddr[0], tmpCliAddr[1]))
            self.__mutexAcc.acquire()
            if (self.__cliAddr):
                logHandler.log.debug("Old client release connection: [%s:%d]" % (self.__cliAddr[0], self.__cliAddr[1]))
            if (self.__cliSock):
                try:
                    logHandler.log.debug("try close old client: [%s:%d]" % (self.__cliAddr[0], self.__cliAddr[1]))
                    self.__cliSock.close()
                    logHandler.log.debug("client closed: [%s:%d]" % (self.__cliAddr[0], self.__cliAddr[1]))
                except Exception as e:
                    logHandler.log.error(repr(e))
                self.__cliSock = None
            self.__cliSock = tmpCliSock
            self.__cliAddr = tmpCliAddr
            self.__cliAlive = True
            time.sleep(1)


class GlobalPlugin(globalPluginHandler.GlobalPlugin):

    global logHandler

    __gestures = {
        "kb:NVDA+shift+v": "toggleSilentMode"
    }

    def __init__(self):
        super(self.__class__, self).__init__()
        synthDriverHandler.initialize()
        self.__synth = synthDriverHandler.getSynth()
        self.__lastTextSend = ""
        self.__server = TCPServer(("127.0.0.1", 10001))
        self.__server.daemon = True
        self.__server.start()
        gui.speechViewer.appendText = self.appendText
        speechViewer.isActive = True

    def script_toggleSilentMode(self, obj):
        synthDriverHandler.getSynthList()
        if (synthDriverHandler.getSynth().name == self.__synth.name):
            synthDriverHandler._curSynth = synthDrivers.silence.SynthDriver()
            state = (u"VLibras Plugin: Modo silencioso ativado")
        else:
            synthDriverHandler.setSynth(self.__synth.name)
            state = (u"VLibras Plugin: Modo silencioso desativado")
        ui.message(state)

    def appendText(self, text):
        try:
            text = u" ".join(text.split())
            if (text and (text != self.__lastTextSend)):
                self.__lastTextSend = text
                if (self.__server.connected()):
                    self.__server.send(text)
        except Exception as e:
            logHandler.log.debug(repr(e))
